from aiohttp import ClientConnectorError, ClientResponse, ClientSession
import os

MBTA_DEFAULT_HOST = "api-v3.mbta.com"

class Auth:
    """Class to make authenticated requests to the MBTA v3 API."""
    
    def __init__(
        self,
        websession: ClientSession,
        api_key: str = None,
        host: str = MBTA_DEFAULT_HOST,
        
    ):
        """Initialize the auth."""
        self.websession = websession
        self.api_key = api_key
        self.host = host
        
        if api_key == None:
            self.api_key = os.environ.get('MBTA_API_KEY')
        
    async def request(
        self, method: str, path: str, payload=None, **kwargs
    ) -> ClientResponse:
        """Make a request."""
        headers = kwargs.get("headers")
        
        if headers is None:
            headers = {}
        else:
            headers = dict(headers)
            
        headers['api_key'] = self.api_key
        
        try:
            response = await self.websession.request(
                method,
                f'https://{self.host}/{path}',
                json=payload,
                **kwargs,
                headers=headers,
            )
            
            data = await response.json()
            
            return_code = data.get("returnCode")
            
            return response
            
        except ClientConnectorError as error:
            raise CannotConnect(error)


from pytest import fixture
from mbtav3.mbta import MBTA
from mbtav3.auth import Auth
from mbtav3.utils import Page, Sort, SortOrder, Filter, build_params
import aiohttp
import pytest
import asyncio
import os

pytestmark = pytest.mark.asyncio

stop = 'place-sull'
route_type = 1
page_offset = 0
page_limit = 3
sort_field = 'departure_time'
sort_order = 'asc'

async def test_predictions():
    session = aiohttp.ClientSession()
    mbta = MBTA(Auth(session,api_key))
    page = Page(page_offset, page_limit)
    order = SortOrder('asc')
    sort = Sort(sort_field, order)
    stopFilter = Filter('stop',stop)
    typeFilter = Filter('route_type', route_type)
    
    params = build_params(stopFilter, typeFilter, page=page, sort=sort)
    
    preds = await mbta.listPredictions(params)
    
    assert isinstance(preds, dict)
    


#import os
#import requests
#
#MBTA_API_KEY = os.environ.get('MBTA_API_KEY', None)
#
#class APIKeyMissingError(Exception):
#    pass
#    
#if MBTA_API_KEY is None:
#    raise APIKeyMissingError (
#        "All methods require an API key. See "
#        "https://api-v3.mbta.com/"
#        "for how to retrieve an authentication token from "
#        "The Massachusetts Bay Transportation Authority"
#    )
#session = requests.Session()
#session.params = {}
#session.params['api_key'] = MBTA_API_KEY
#
#from .line import Line
#from .stop import Stop
#from .route import Route


import enum
import typing

Page = typing.NamedTuple('Page', [('offset', int), ('limit', int)])
Point = typing.NamedTuple('Point', [('latitude', float), ('longitude', float)])


class SortOrder(enum.Enum):
    asc = 'asc'
    desc = 'desc'


Sort = typing.NamedTuple('Sort', [('field', str), ('order', SortOrder)])
Filter = typing.NamedTuple('Filter', [('field', str), ('value', typing.Any)])
MultiFilter = typing.NamedTuple('MultiFilter', [('field', str), ('values', typing.Iterable[str])])


def make_multifilter(key: str, value):
    if value is None:
        return None

    to_filter = [value]
    if not isinstance(value, str):
        try:
            to_filter = list(x for x in value if x is not None)
        except TypeError:
            pass

    if len(to_filter) == 0:
        return None

    return MultiFilter(field=key, values=to_filter)


class WheelchairAccess(enum.Enum):
    Unknown = 0
    Accessible = 1
    Inaccessible = 2
    
def build_params(
        *args,
        include: typing.Iterable[str]=None,
        location: Point=None,
        page: Page=None,
        sort: Sort=None
) -> typing.Dict[str, str]:
    params = {}

    for arg in args:
        if arg is None:
            continue
        if isinstance(arg, Filter):
            params[f'filter[{arg.field}]'] = arg.value
        elif isinstance(arg, MultiFilter):
            params[f'filter[{arg.field}]'] = ','.join(str(x) for x in arg.values)
        elif isinstance(arg, tuple):
            params[arg[0]] = arg[1]

    if location:
        params['filter[latitude]'] = location.latitude
        params['filter[longitude]'] = location.longitude

    if page:
        params['page[offset]'] = page.offset
        params['page[limit]'] = page.limit

    if sort:
        params['sort'] = f'{sort.field}' if sort.order.asc else f'-{sort.field}'

    if include:
        params['include'] = ','.join(include)

    return params


from .auth import Auth
from .const import *

class MBTA:
    def __init__(self, auth):
        self.auth = auth
        
    async def getStop(self, id, payload=None):
        """Get the details for a stop."""
        response = await self.auth.request(
            "get", f'{ENDPOINT_STOPS}/{id}', payload)
        return await response.json()
        
    async def getTrip(self, id, payload=None):
        """Get the details for a trip."""
        response = await self.auth.request(
            "get", f'{ENDPOINT_TRIPS}/{id}', payload)
        return await response.json()
        
    async def getRoute(self, id, payload=None):
        """Get the details for a route."""
        response = await self.auth.request(
            "get", f'{ENDPOINT_ROUTES}/{id}', payload)
        return await response.json()
        
    async def listPredictions(self, payload):
        """Get a list of Predictions."""
        response = await self.auth.request(
            "get", f'{ENDPOINT_PREDICTIONS}', params=payload
        )
        return await response.json()


from setuptools import setup, find_packages

setup(
    name='mbtav3',
    version='0.2',
    packages=find_packages(exclude=['tests']),
    url='https://github.com/djsv23/mbtav3',
    license='MIT',
    author='Daniel Steiner',
    author_email='djsteiner93@gmail.com',
    description='Python library for MBTA v3 realtime API',
    classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries'
    ],
    install_requires=[
        'aiohttp'
    ]
)
